import yaml
from ffctl.commands.command_base import CommandBase, LoadVariables
from ffctl.commands.lint import gitlab_lint, lint_status


class JsonnetCmd(CommandBase):
    name = 'gen'
    help_message = "Generate the .gitlab-ci.yml from the jsonnet"

    def __init__(self, options):
        super(JsonnetCmd, self).__init__(options)
        self.variables = options.variables
        self.host = options.host
        self.no_lint = options.no_lint
        self.dry = options.dry
        if isinstance(options.filepath, list):
            self.filepath = options.filepath[0]
        else:
            self.filepath = options.filepath
        self.result = ''

    @classmethod
    def _add_arguments(cls, parser):
        parser.add_argument("-x", "--variables", help="variables", default={}, action=LoadVariables)
        parser.add_argument('filepath', nargs='?', default=[".gitlab-ci.jsonnet"], help="jsonnet file to render")
        parser.add_argument('--no-lint', action="store_true", default=False, help="lint the output")
        parser.add_argument('--dry', action="store_true", default=False, help="don't write into .gitlab-ci.yml")
        parser.add_argument("-H", "--host", help="gitlab host (for lint)", default="https://gitlab.com")

    def _call(self):
        from ffctl.render_jsonnet import RenderJsonnet
        r = RenderJsonnet(manifestpath=self.filepath)
        tla_codes = self.variables
        p = open(self.filepath).read()
        gitlabcidict = r.render_jsonnet(p, tla_codes=tla_codes)
        gitlabciyaml = yaml.safe_dump(gitlabcidict, default_flow_style=False, indent=2, width=200)

        if not self.no_lint:
            lint_result = gitlab_lint(self.host, gitlabciyaml)
            if not lint_status(lint_result):
                self.result += yaml.safe_dump(lint_result, default_flow_style=False)
                return
        if not self.dry:
            with open(".gitlab-ci.yml", "wb") as f:
                f.write("# Generated from .gitlab-ci.jsonnet \n"
                        "#     DO NOT EDIT THIS FILE BY HAND -- YOUR CHANGES WILL BE OVERWRITTEN \n"
                        "---\n")
                f.write(gitlabciyaml)
            self.result += "-> .gitlab-ci.yml updated"
        else:
            self.result += gitlabciyaml

    def _render_dict(self):
        return self.result

    def _render_console(self):
        return self.result
